/*
Generic RC Trailer and Accessory Code Version 1.0
 This code was written by Oisin O'Conchubhair for the website www.rctractors.net
 This is the generic code used to control homemade RC Tractors.
 The code is intended to be used with an Arduino Pro Mini
 Circuit diagram can be found ar www.rctractors.net
 Pin Assignment
 D0 Serial RX
 D1 Serial TX
 D2 Left Indicator
 D3 Head Lights
 D4 Right Indicator
 D5 Drive Motor Speed Control
 D6 Brake Lights
 D7 Drive Motor Direction Control 1
 D8 Drive Motor Direction Control 2
 D9 Three-Point Linkage Servo Signal
 D10 Steering Servo Signal
 D11 Accessory Servo Signal
 D12 Accessory Serial TX Signal
 D13 Spot Lights
 A0 Beacon 1
 A1 Beacon 2
 A2 Not assigned
 A3 Not assigned
 A4 Not assigned
 A5 Not assigned
 */

// This section describes the vehicle
byte servo1_min = 0x50; // Servo 1 Minimum
byte servo1_max = 0x80; // Servo 1 Maximum
byte servo2_min = 0x32; // Servo 2 Minimum
byte servo2_max = 0x80; // Servo 2 Maximum
byte servo3_min = 0x50; // Servo 3 Minimum
byte servo3_max = 0xAA; // Servo 3 Maximum
byte servo4_min = 0x00; // Servo 4 Minimum
byte servo4_max = 0xFF; // Servo 4 Maximum
byte servo5_min = 0x00; // Servo 5 Minimum
byte servo5_max = 0xFF; // Servo 5 Maximum

#define SOP '<'
#define EOP '>'

bool started = false;
bool ended = false;

uint8_t buf[16]; // Array to store recieved data
uint8_t command[16]; // Array to store recieved data

byte index;

// Variables used in the program
byte Data_1 = 0x00; // Stores the first byte in the command string
byte Command_Val = 0x00; // Stores the command value
byte Data_Val = 0x00; // Stores the data value
byte Old_Data_Val = 0x00; // Stores the data value
byte servo_val = 0x00; // Stores the servo position value
int Door_Button;
int Door_Open = 0;
int Lock_Button;
int Lock_Open = 0;
int Data_Counter = 0;
int Load = 2;
int Pickup = 0;
int Old_Func_Val = 0;
int Func_Val = 0;
int Reel_Pos = 1;


#include <Servo.h> 
 
Servo servo1;  // Pick up reel servo 
Servo servo2;  // Rear door servo 
Servo servo3;  // Second bale release servo 

void setup() 
{
  Serial.begin(9600);

  pinMode(5, INPUT_PULLUP); // Motor Direction 1
  pinMode(6, INPUT_PULLUP); // Motor Direction 2
  servo1.attach(7);  // attaches the servo on pin 9 to the servo object 
  servo2.attach(8);  // attaches the servo on pin 9 to the servo object 
  servo3.attach(9);  // attaches the servo on pin 9 to the servo object 

  servo1.write(0x78);      // Start with the pick up reel raised
  //30 down 78 up
  servo2.write(0x32);      // Start with door closed
  //80 open 32 closed
  servo3.write(0x50);      // Start with inner bale locked
//50 closed 95 open
 }

void loop()
{
  check_button();
  check_serial();

  if(Reel_Pos == 1 && bitRead(command[10], 0) == 0){
             servo1.write(servo1_min);      // Open Rear Door
  	  	  	  Reel_Pos = 0;
  }
else if  (Reel_Pos == 0 && bitRead(command[10], 0) == 1){
      servo1.write(servo1_max);      // Open Rear Door
Reel_Pos = 1;
  }


  if(Load == 2 && bitRead(command[10], 1) == 1){
             servo2.write(servo2_max);      // Open Rear Door
           Load = 1;
          Door_Open=1;
    }
  else if(Load == 1 && bitRead(command[10], 1) == 0){
             servo2.write(servo2_min);      // Open Rear Door
             servo3.write(servo3_max);      // Open Rear Door
             Door_Open=0;
             Lock_Open=1;
    }

  else if(Load == 1 && bitRead(command[10], 1) == 1 && Door_Open == 0){
  	    servo2.write(servo2_max);      // Open Rear Door
  	    Door_Open=1;
  	    Load = 0;
  	  }

  else if(Load == 0 && bitRead(command[10], 1) == 0){
  	               servo2.write(servo2_min);      // Open Rear Door
  	   Door_Open=0;
  	  }


}

void check_button(){
        Door_Button = digitalRead(5);
      Lock_Button = digitalRead(6);


  if (Door_Button==LOW)
  {
    if (Door_Open != 1){
      Door_Open=1;  // Command to turn on right indicator
        servo2.write(servo2_max);      // Open Rear Door

    }
    else {
      Door_Open=0;  // Command to turn on right indicator
      servo2.write(servo2_min);      // Close Rear Door
      Load = 2;
    }

    // Wait for switch to be released
    while (Door_Button == LOW)
    {
      Door_Button = digitalRead(5);
    }
     }
     
      if (Lock_Button==LOW)
  {
    if (Lock_Open != 1 && Door_Open == 1){
      Lock_Open=1;  // Command to turn on right indicator
        servo3.write(servo3_max);      // Open Rear Door

    }
    else {
      Lock_Open=0;  // Command to turn on right indicator
      servo3.write(servo3_min);      // Close Rear Door
      Load = 1;
    }

    // Wait for switch to be released
    while (Lock_Button == LOW)
    {
      Lock_Button = digitalRead(6);
    }
     }
}

void check_serial(){  
	  if (Serial.available()) { // Check if serial data is available
// Read all serial data available, as fast as possible
while(Serial.available() > 0)
{
 char inChar = Serial.read();
 if(inChar == SOP)
 {
    index = 0;
    buf[index] = '\0';
    started = true;
    ended = false;
 }
 else if(inChar == EOP)
 {
	if(index == 11){
		for (int i = 0; i < 12; i++) {
		command[i]=buf[i];
		}
	}
	else if(index == 2){
		 	   	  	 		for (int i = 0; i < 3; i++) {
		 	   	  	 		command[i]=buf[i];
		 	   	  	 		}
		 	}
    ended = true;
    break;
 }
 else
 {
   if(index < 15)
   {
     buf[index] = inChar;
     index++;
     buf[index] = '\0';
   }
 }
}

// We are here either because all pending serial
// data has been read OR because an end of
// packet marker arrived. Which is it?
if(started && ended)
{
 // The end of packet marker arrived. Process the packet

 // Reset for the next packet
 started = false;
 ended = false;
 index = 0;
 buf[index] = '\0';
}


}
}
