/*

*/

// This section describes the vehicle

// Variables used in the program
byte ID = 1; // Vehicle ID Number
int Active = 1; // Variable to determine if the tractor is in active
int Last_Data_Timer = 0;
int Controller_Locked = 0;
int Receive_Count = 0;

int Controller_ID = 0;

uint8_t buf[16]; // Array to store recieved data

#include <SPI.h>
#include "RCTractorGuyRF24.h"
#include "RCTractorGuynRF24L01.h"
#include "RCTractorGuy.h"

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10
RF24 radio(9, 10);

// Initialise Tractor
/*(Motor_1_PWM, Motor_1_Dir_1, Motor_1_Dir_2, Motor_2_PWM, Motor_2_Dir_1, Motor_2_Dir_2,
Steering, Rear_Link, Front_Link, Left_Ind, Right_Ind, Head_Lights, Tail_Lights, Work_Lights,
Accessory_Lights, Left_Beacon, Right_Beacon)*/
RCTractor JD6920S(5, 0, 0, 6, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4);

// Controller Address
const uint64_t Controller[5] = { 0xE8E8F0F0E1LL, 0xE8E8F0F0E2LL, 0xE8E8F0F0E3LL, 0xE8E8F0F0E4LL, 0xE8E8F0F0E5LL};
const uint64_t Controller_Listen[5] = { 0x3A3A3A3AD1LL, 0x3A3A3A3AD2LL, 0x3A3A3A3AD3LL, 0x3A3A3A3AD4LL, 0x3A3A3A3AD5LL };

void setup(void)
{
  // Start the NRF24L01
  radio.begin();


  // Initialise the library
  JD6920S.begin();

  // Set the servo limits
  // (Steering Servo Min, Steering Servo Max, Rear Link Servo Min, Rear Link Servo Max, Front Link Servo Min, Front Link Servo Max)
  JD6920S.SetServoLimits(60, 120, 0x10, 0x70, 0x00, 0xFF);

  // Set the motor configuration
  // (Motor configuration 1 = 1 motor or 2 = 2 motors or 3 = tracks, Do you use a PCF8574 0=no or 1=yes)
  JD6920S.MotorConfiguration(2, 1);

  radio.openReadingPipe(1, Controller[0]); // Set Address of Controller 1
  radio.openReadingPipe(2, Controller[1]); // Set Address of Controller 2
  radio.openReadingPipe(3, Controller[2]); // Set Address of Controller 1
  radio.openReadingPipe(4, Controller[3]); // Set Address of Controller 2
  radio.openReadingPipe(5, Controller[4]); // Set Address of Controller 2

  radio.startListening();

    Serial.begin(9600);
}

void loop(void)
{

    buf[0] = 0;

    if ( radio.available() )
    {
        // Fetch the payload.
       radio.read( buf, sizeof(buf));

      // We recieved a string of ten bytes, four ID, one command and six data values but they may not be for this tractor
      if (buf[0] == ID) { // Confirm that the correct vehicle ID has been recieved


      int pipe_num = buf[11];
        Serial.print(pipe_num);

      long Vehicle_Voltage = JD6920S.ReadVcc();

     // First, stop listening so we can talk
      radio.stopListening();

      // Open the correct pipe for writing
      radio.openWritingPipe(Controller_Listen[pipe_num]);

      // Send battery voltage back.
      radio.write( &Vehicle_Voltage, sizeof(Vehicle_Voltage) );

      // Now, resume listening so we catch the next packets.
      radio.startListening();

        JD6920S.Drive(buf[2], buf[3]);
        JD6920S.LiftingArms(buf[5], buf[6]);
        JD6920S.Lights(buf[9]);        // Update the lights
        JD6920S.Trailer(buf);	// Send Command to any trailers

        Last_Data_Timer = 10000;
        Active = 1;
      }
    }
    else if (Last_Data_Timer != 0) {
      Last_Data_Timer = Last_Data_Timer - 1;
    }
    else if (Last_Data_Timer == 0 && Active == 1) {
      Active = 0;
      JD6920S.Lights(0x03);
      JD6920S.Drive(125, 125);
    }

    JD6920S.Timer();


    /* if (Active == 1) {
      }
      else if (Controller_Locked != 0) {
       All_Controllers();
       Controller_Locked = 0;
      }*/

}


